with Ada.Text_IO;
with Sorting;

use Ada.Text_IO;
use Sorting;

procedure Main is

    procedure Assert(Condition: Boolean; Error_String: String) is
    begin
        if not Condition then
            Put_Line(Error_String);
        end if;
    end Assert;

    -- ---------------------------------------------------------------

    From_List:  constant Natural_Array := (Natural'First, 10, Natural'Last - 1000);
    Mixed_List: constant Natural_Array := (23, 37, 1, 12, 25, 9, 6, 18, 42, 13);
    
    -- ---------------------------------------------------------------

    function Create_Array(Num_Items: Natural; From: Natural) 
        return Natural_Array is
        A: constant Natural_Array(From .. From + Num_Items - 1) := (others => 0);
    begin
        return A;
    end Create_Array;

    -- ---------------------------------------------------------------

    procedure Fill_Array_Sorted(A: in out Natural_Array) is
    begin
        for I in A'range loop
            A(I) := I;
        end loop;
    end Fill_Array_Sorted;

    -- ---------------------------------------------------------------

    procedure Fill_Array_Mixed(A: in out Natural_Array) is
        Offset: Natural;
        J: Natural := 0;
    begin
        for I in A'range loop
            Offset := I mod Mixed_List'Length;
            J      := Integer((I - A'First) / Mixed_List'Length);
            A(I)   := Offset + J;
        end loop;
    end Fill_Array_Mixed;

    -- ---------------------------------------------------------------

    procedure Fill_Array_With_Equal_Items(A: in out Natural_Array) is
    begin
        for I in A'range loop
            A(I) := 42;
        end loop;
    end Fill_Array_With_Equal_Items;

    -- ---------------------------------------------------------------

    procedure Fill_Array_Reverse_Sorted(A: in out Natural_Array) is
    begin
        for I in A'range loop
            A(I) := A'Last - I + A'First;
        end loop;
    end Fill_Array_Reverse_Sorted;

    -- ---------------------------------------------------------------

    function Are_Sets_Equal(A: Natural_Array; B: Natural_Array) 
        return Boolean is
    begin
        if A'First /= B'First or else A'Last /= B'Last then
            return False;
        end if;

        for I in A'range loop
            if not Contains(B, A(I)) then
                return False;
            end if;
        end loop;

        return True;
    end Are_Sets_Equal;

    -- ---------------------------------------------------------------

    procedure Test_Array(Num_Items: Natural) is
    begin
        for I in From_List'range loop
            declare
                From: constant Natural := From_List(I);
                A: Natural_Array := Create_Array(Num_Items, From);
                A_Old: Natural_Array := Create_Array(Num_Items, From);
            begin
                Fill_Array_Sorted(A);
                Fill_Array_Sorted(A_Old);
                Selection_Sort(A);
                Assert(Is_Sorted(A), "Array was not sorted when filled sorted");
                Assert(Are_Sets_Equal(A, A_Old), 
                    "Array lacks elements after sort when filled sorted");

                Fill_Array_Mixed(A);
                Fill_Array_Mixed(A_Old);
                Selection_Sort(A);
                Assert(Is_Sorted(A), "Array was not sorted when filled mixed");
                Assert(Are_Sets_Equal(A, A_Old), 
                    "Array lacks elements after sort when filled mixed");

                Fill_Array_With_Equal_Items(A);
                Fill_Array_With_Equal_Items(A_Old);
                Selection_Sort(A);
                Assert(Is_Sorted(A), 
                    "Array was not sorted when filled with equal items");
                Assert(Are_Sets_Equal(A, A_Old), "Array lacks elements "
                    & "after sort when filled with equal items");

                Fill_Array_Reverse_Sorted(A);
                Fill_Array_Reverse_Sorted(A_Old);
                Selection_Sort(A);
                Assert(Is_Sorted(A), 
                    "Array was not sorted when filled in reverse order");
                Assert(Are_Sets_Equal(A, A_Old), "Array lacks elements after "
                    & "sort when filled in reverse order");
            end;
        end loop;
    end Test_Array;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Empty is
    begin
        Test_Array(0);
    end Test_Sort_Empty;

    -- ---------------------------------------------------------------
    
    procedure Test_Sort_Single_Item is
    begin
        Test_Array(1);
    end Test_Sort_Single_Item;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Some_Items is
    begin
        Test_Array(10);
    end Test_Sort_Some_Items;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Many_Items is
    begin
        Test_Array(1000);
    end Test_Sort_Many_Items;

begin
    Test_Sort_Empty;
    Test_Sort_Single_Item;
    Test_Sort_Some_Items;
    Test_Sort_Many_Items;
end Main;
