with AUnit.Assertions;
with Sorting;

use AUnit.Assertions;
use Sorting;

package body Sorting_Tests is

    From_List:  constant Natural_Array 
        := (Natural'First, 10, Natural'Last - 1000);
    Mixed_List: constant Natural_Array := (23, 37, 1, 12, 25, 9, 6, 18, 42, 13);
    
    -- ---------------------------------------------------------------

    function Create_Array(Num_Items: Natural; From: Natural) 
        return Natural_Array is
        A: constant Natural_Array(From .. From + Num_Items - 1) 
            := (others => 0);
    begin
        return A;
    end Create_Array;

    -- ---------------------------------------------------------------

    procedure Fill_Array_Sorted(A: in out Natural_Array) is
    begin
        for I in A'range loop
            A(I) := I;
        end loop;
    end Fill_Array_Sorted;

    -- ---------------------------------------------------------------

    procedure Fill_Array_Mixed(A: in out Natural_Array) is
        Offset: Natural;
        J: Natural := 0;
    begin
        for I in A'range loop
            Offset := I mod Mixed_List'Length;
            J      := Integer((I - A'First) / Mixed_List'Length);
            A(I)   := Offset + J;
        end loop;
    end Fill_Array_Mixed;

    -- ---------------------------------------------------------------

    procedure Fill_Array_With_Equal_Items(A: in out Natural_Array) is
    begin
        for I in A'range loop
            A(I) := 42;
        end loop;
    end Fill_Array_With_Equal_Items;

    -- ---------------------------------------------------------------

    procedure Fill_Array_Reverse_Sorted(A: in out Natural_Array) is
    begin
        for I in A'range loop
            A(I) := A'Last - I + A'First;
        end loop;
    end Fill_Array_Reverse_Sorted;

    -- ---------------------------------------------------------------

    function Is_Sorted(A: Natural_Array) return Boolean is
    begin
        for I in A'First .. A'Last-1 loop
            if A(I) > A(I+1) then
                return False;
            end if;
        end loop;

        return True;
    end Is_Sorted;

    -- ---------------------------------------------------------------

    function Contains(A: Natural_Array; Item: Natural) return Boolean is
    begin
        for I in A'range loop
            if A(I) = Item then
                return True;
            end if;
        end loop;

        return False;
    end Contains;

    -- ---------------------------------------------------------------

    function Are_Sets_Equal(A: Natural_Array; B: Natural_Array) 
        return Boolean is
    begin
        if A'First /= B'First or else A'Last /= B'Last then
            return False;
        end if;

        for I in A'range loop
            if not Contains(B, A(I)) then
                return False;
            end if;
        end loop;

        return True;
    end Are_Sets_Equal;

    -- ---------------------------------------------------------------

    procedure Test_Array_Sorted(Num_Items: Natural) is
    begin
        for I in From_List'range loop
            declare
                From: constant Natural := From_List(I);
                A: Natural_Array := Create_Array(Num_Items, From);
                A_Old: Natural_Array := Create_Array(Num_Items, From);
            begin
                Fill_Array_Sorted(A);
                Fill_Array_Sorted(A_Old);
                Selection_Sort(A);
                Assert(Is_Sorted(A), "Array was not sorted when filled sorted");
                Assert(Are_Sets_Equal(A, A_Old), 
                    "Array lacks elements after sort when filled sorted");
            end;
        end loop;
    end Test_Array_Sorted;

    -- ---------------------------------------------------------------

    procedure Test_Array_Mixed(Num_Items: Natural) is
    begin
        for I in From_List'range loop
            declare
                From: constant Natural := From_List(I);
                A: Natural_Array := Create_Array(Num_Items, From);
                A_Old: Natural_Array := Create_Array(Num_Items, From);
            begin
                Fill_Array_Mixed(A);
                Fill_Array_Mixed(A_Old);
                Selection_Sort(A);
                Assert(Is_Sorted(A), "Array was not sorted when filled mixed");
                Assert(Are_Sets_Equal(A, A_Old), 
                    "Array lacks elements after sort when filled mixed");
            end;
        end loop;
    end Test_Array_Mixed;

    -- ---------------------------------------------------------------

    procedure Test_Array_With_Duplicates(Num_Items: Natural) is
    begin
        for I in From_List'range loop
            declare
                From: constant Natural := From_List(I);
                A: Natural_Array := Create_Array(Num_Items, From);
                A_Old: Natural_Array := Create_Array(Num_Items, From);
            begin
                Fill_Array_With_Equal_Items(A);
                Fill_Array_With_Equal_Items(A_Old);
                Selection_Sort(A);
                Assert(Is_Sorted(A), 
                    "Array was not sorted when filled with duplicates");
                Assert(Are_Sets_Equal(A, A_Old), "Array lacks elements "
                    & "after sort when filled with duplicates");
            end;
        end loop;
    end Test_Array_With_Duplicates;

    -- ---------------------------------------------------------------

    procedure Test_Array_Reverse_Sorted(Num_Items: Natural) is
    begin
        for I in From_List'range loop
            declare
                From: constant Natural := From_List(I);
                A: Natural_Array := Create_Array(Num_Items, From);
                A_Old: Natural_Array := Create_Array(Num_Items, From);
            begin
                Fill_Array_Reverse_Sorted(A);
                Fill_Array_Reverse_Sorted(A_Old);
                Selection_Sort(A);
                Assert(Is_Sorted(A), 
                    "Array was not sorted when filled in reverse order");
                Assert(Are_Sets_Equal(A, A_Old), "Array lacks elements after "
                    & "sort when filled in reverse order");
            end;
        end loop;
    end Test_Array_Reverse_Sorted;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Single_Item(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array_Sorted(1);
    end Test_Sort_Single_Item;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Some_Items_Sorted(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array_Sorted(10);
    end Test_Sort_Some_Items_Sorted;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Some_Items_Mixed(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array_Mixed(10);
    end Test_Sort_Some_Items_Mixed;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Some_Items_With_Duplicates(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array_With_Duplicates(10);
    end Test_Sort_Some_Items_With_Duplicates;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Some_Items_Reverse_Sorted(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array_Reverse_Sorted(10);
    end Test_Sort_Some_Items_Reverse_Sorted;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Many_Items_Sorted(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array_Sorted(100);
    end Test_Sort_Many_Items_Sorted;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Many_Items_Mixed(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array_Mixed(100);
    end Test_Sort_Many_Items_Mixed;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Many_Items_With_Duplicates(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array_With_Duplicates(100);
    end Test_Sort_Many_Items_With_Duplicates;

    -- ---------------------------------------------------------------

    procedure Test_Sort_Many_Items_Reverse_Sorted(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array_Reverse_Sorted(100);
    end Test_Sort_Many_Items_Reverse_Sorted;

    -- ---------------------------------------------------------------

    procedure Register_Tests(T: in out Test) is
        use AUnit.Test_Cases.Registration;
    begin
        Register_Routine(T, Test_Sort_Single_Item'Access, 
            "Test_Sort_Single_Item");
        Register_Routine(T, Test_Sort_Some_Items_Sorted'Access, 
            "Test_Sort_Some_Items_Sorted");
        Register_Routine(T, Test_Sort_Some_Items_Mixed'Access, 
            "Test_Sort_Some_Items_Mixed");
        Register_Routine(T, Test_Sort_Some_Items_With_Duplicates'Access, 
            "Test_Sort_Some_Items_With_Duplicates");
        Register_Routine(T, Test_Sort_Some_Items_Reverse_Sorted'Access, 
            "Test_Sort_Some_Items_Reverse_Sorted");
        Register_Routine(T, Test_Sort_Many_Items_Sorted'Access, 
            "Test_Sort_Many_Items_Sorted");
        Register_Routine(T, Test_Sort_Many_Items_Mixed'Access, 
            "Test_Sort_Many_Items_Mixed");
        Register_Routine(T, Test_Sort_Many_Items_With_Duplicates'Access, 
            "Test_Sort_Many_Items_With_Duplicates");
        Register_Routine(T, Test_Sort_Many_Items_Reverse_Sorted'Access, 
            "Test_Sort_Many_Items_Reverse_Sorted");
    end Register_Tests;
    
    -- ---------------------------------------------------------------

    function Name(T: Test) return Test_String is
        pragma Unreferenced(T);
    begin
        return Format("Sorting_Tests");
    end Name;
    
end Sorting_Tests;
