with Ada.Text_IO;
with AUnit.Assertions;
with Parallel_Algorithms;

use Ada.Text_IO;
use AUnit.Assertions;

package body Parallel_Algorithms_Tests is

    package Integer_Algorithms is new Parallel_Algorithms(
        Item_Type  => Integer,
        "+"        => "+"
    );
    use Integer_Algorithms;

    -- ---------------------------------------------------------------

    function Create_Array(Num_Items: Natural) return Array_Access_Type is
        A: constant Array_Access_Type := 
            new Array_Type(0 .. Integer(Num_Items)-1);
    begin
        return A;
    end Create_Array;

    -- ---------------------------------------------------------------

    procedure Fill_Array_Sorted(A: Array_Access_Type) is
    begin
        for I in A.all'range loop
            A.all(I) := 2;
        end loop;
    end Fill_Array_Sorted;

    -- ---------------------------------------------------------------

    function Sum(A: Array_Access_Type) return Natural is
        Result: Natural := 0;
    begin
        for I in A.all'range loop
            Result := Result + A.all(I);
        end loop;

        return Result;
    end Sum;

    -- ---------------------------------------------------------------

    procedure Test_Array(Num_Items: Natural) is
        A: constant Array_Access_Type := Create_Array(Num_Items);
        Actual:   Natural := 0;
        Expected: Natural := 0;
    begin
        Put_Line("Filling test array");

        Fill_Array_Sorted(A);

        Put_Line("Filled test array");
        Put_Line("Summing test array sequentially");

        Expected := Sum(A);

        Put_Line("Summed test array sequentially");

        Parallel_Sum(A, Actual);
        Assert(Actual = Expected, "Expected " & Expected'Img 
            & " but was " & Actual'Img);
    end Test_Array;

    -- ---------------------------------------------------------------

    procedure Test_Sum_Empty(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array(0);
    end Test_Sum_Empty;

    -- ---------------------------------------------------------------
    
    procedure Test_Sum_Single_Item(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array(1);
    end Test_Sum_Single_Item;

    -- ---------------------------------------------------------------

    procedure Test_Sum_Some_Items(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array(10);
    end Test_Sum_Some_Items;

    -- ---------------------------------------------------------------

    procedure Test_Sum_Many_Items(T: in out Test_Case'Class) is
        pragma Unreferenced(T);
    begin
        Test_Array(500_000_000);
    end Test_Sum_Many_Items;

    -- ---------------------------------------------------------------

    procedure Register_Tests(T: in out Test) is
        use AUnit.Test_Cases.Registration;
    begin
        Register_Routine(T, Test_Sum_Empty'Access, 
            "Test_Sum_Empty");
        Register_Routine(T, Test_Sum_Single_Item'Access, 
            "Test_Sum_Single_Item");
        Register_Routine(T, Test_Sum_Some_Items'Access, 
            "Test_Sum_Some_Items");
        Register_Routine(T, Test_Sum_Many_Items'Access, 
            "Test_Sum_Many_Items");
    end Register_Tests;
    
    -- ---------------------------------------------------------------

    function Name(T: Test) return Test_String is
        pragma Unreferenced(T);
    begin
        return Format("Parallel_Algorithms_Tests");
    end Name;
    
end Parallel_Algorithms_Tests;
