with Ada.Directories;
with Ada.Sequential_IO;

package body Array_IO is
	use Ada.Directories;
	
	package Item_IO is new Ada.Sequential_IO(Item_Type);
	use Item_IO;
	
    -- ---------------------------------------------------------------

	function Get_Num_Items_In_File(File_Path: String) return Natural is
		Bits_Per_Byte: constant Natural := 8;
	begin
		return Natural(Size(File_Path)) * Bits_Per_Byte / Item_Type'Size;
	end Get_Num_Items_In_File;
	
    -- ---------------------------------------------------------------

    function Read_File(File_Path: String) return Array_Access_Type is
		File: File_Type;
        File_Size: constant Natural := Get_Num_Items_In_File(File_Path);
        A: constant Array_Access_Type := new Array_Type(0 .. File_Size - 1);
		Item: Item_Type;
    begin
		Open(File, In_File, File_Path);
        
		for I in A'Range loop
			Read(File, Item);
			A.all(I) := Item;
		end loop;
		
		Close(File);
		return A;
    end Read_File;
    
    -- ---------------------------------------------------------------

    procedure Write_To_File(File_Path: String; A: Array_Type) is
        File: File_Type;
    begin
        if not Exists(File_Path) then
            Create(File, Out_File, File_Path);
        else
            Open(File, Out_File, File_Path);
        end if;
        
        for I in A'Range loop
            Write(File, A(I));
        end loop;
		
        Close(File);
    end Write_To_File;
	
end Array_IO;
