with Ada.Command_Line;
with Ada.Numerics.Discrete_Random;
with Ada.Sequential_IO;
with Ada.Text_IO;

procedure Write_Random_Numbers is
    
    Missing_Arguments_Exception: exception;
    
    package Integer_IO is new Ada.Sequential_IO(Integer);
    use Integer_IO;
    
    package Random_Generators is new Ada.Numerics.Discrete_Random(Integer);
    use Random_Generators;
    
    -- ---------------------------------------------------------------

    procedure Print_Usage is
        use Ada.Text_IO;
    begin
        Put_Line("Stores a sequence of n random integers into a file.");
        Put_Line("Usage: ./write_random_numbers <n> <output_path>");
        Put_Line("- <n>: #Random numbers to write.");
        Put_Line("- <output_path>: Relative file path to write to.");
    end Print_Usage;
    
    -- ---------------------------------------------------------------

    function Read_Num_Numbers return Natural is 
        use Ada.Command_Line;
    begin
        if Argument_Count < 1 then
            Print_Usage;
            raise Missing_Arguments_Exception;
        else
            return Natural'Value(Argument(1));
        end if;
    end Read_Num_Numbers;

    -- ---------------------------------------------------------------

    function Read_File_Path return String is 
        use Ada.Command_Line;
    begin
        if Argument_Count < 2 then
            Print_Usage;
            raise Missing_Arguments_Exception;
        else
            return Argument(2);
        end if;
    end Read_File_Path;

    -- ---------------------------------------------------------------

    procedure Write_To_File(File_Path: String; Num_Numbers: Natural) is
        Current: Integer;
        Output_File: File_Type;
        RNG: Generator;
    begin
        Reset(RNG);
        Create(File => Output_File, Mode => Out_File, Name => File_Path);

        for I in 1..Num_Numbers loop
            Current := Random(RNG);
            Write(Output_File, Current);
        end loop;
        
        Close(Output_File);
    end Write_To_File;     

    -- ---------------------------------------------------------------

    Output_File_Path: constant String := Read_File_Path;
    Num_Numbers: constant Natural := Read_Num_Numbers;
begin
    Write_To_File(Output_File_Path, Num_Numbers);
end Write_Random_Numbers;
